use diesel::pg::PgConnection;
use diesel::prelude::*;
use dotenvy::dotenv;
use miniweb::users::models::{NewUser, User};
use std::env;
use std::io::stdin;

pub fn establish_connection() -> PgConnection {
    dotenv().ok();

    let database_url = env::var("DATABASE_URL").expect("DATABASE_URL must be set");
    PgConnection::establish(&database_url)
        .unwrap_or_else(|_| panic!("Error connecting to {}", database_url))
}

pub fn create_user(conn: &mut PgConnection, username: &str) -> User {
    use miniweb::schema::users;

    let new_user = NewUser { username };

    diesel::insert_into(users::table)
        .values(&new_user)
        .returning(User::as_returning())
        .get_result(conn)
        .expect("Error saving new user")
}

fn main() {
    let connection = &mut establish_connection();

    let mut username = String::new();

    println!("Enter username for new user:");
    stdin().read_line(&mut username).unwrap();
    let username = username.trim_end(); // Remove the trailing newline

    let user = create_user(connection, username);
    println!("\nSaved user {} with id {}", username, user.id);
}
